#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <unistd.h>
#include <dmedia/audio.h>
#include <errno.h>
#include "client.h"
#include "audioplay.h"
#include "adpcm.h"
#include "networkio.h"

url_info **Urls;
static long blockSize;
header *Info;
int urlCount;
static int procFlag;

static ALport openPort(char *name, header *info);
static void Abandon(char *s);
static Err_mesg(char *s);
static void slideshow(int *numUrls);
static void launcher(void);
static void urlOpener(int signo);
static void timer(int *numUrls);
static void start_play(int signo);
static int findproc(char *s);

void Play(int sockfd, url_info **urls, int numUrls, header *info)
{
  int slide, Read, pid, i;
  ALport output;
  char inbuf[2048];
  short outbuf[4096];
  struct adpcm_state decodeState;

  Urls = urls; 

  Info = info;

  signal(SIGUSR2, start_play);
  slide = sproc((void *) slideshow, PR_SADDR, (int *)&numUrls);
  if (slide == -1)
    Abandon("slide:sproc error");

  output = openPort("audio_play", info);
  
  sigpause(SIGUSR2);

  while (1) {
    
/*    Read = readn(sockfd,inbuf, (int) (blockSize * sizeof(char))); */
    Read = readNetwork(inbuf, (int) (blockSize * sizeof(char)));
    if (Read < 0)
      Abandon("play:read error from network");
    else
      if (Read == 0) /* EOF */
	break;
      else {
	adpcm_decoder(inbuf, outbuf, 2*Read, &decodeState);
	while (ALgetfillable(output) < (long) (2 * Read))
	  ;
	ALwritesamps(output, (void *) outbuf, (long) (2*Read));
      }
  }
  
  while (ALgetfilled(output))
    ;
  ALcloseport(output);
  fprintf(stdout, "play finished\n");
  
  if (pid = findproc("netscape"))
    kill(pid, SIGKILL); 

  system("clear");
}

static void start_play(int signo)
{
  fprintf(stdout, " caught signal %d\n", signo);
}

static void slideshow(int *numUrls)
{
  pid_t self;
  int launch, time;
 
  self = getpid();
  launch = sproc((void *)launcher, PR_SADDR);
  if (launch == -1)
    Abandon("slideshow: sproc error");
  
  sleep(2);
  time = sproc((void *) timer, PR_SADDR, (int *)numUrls);
  if (time == -1)
    Abandon("slideshow: sproc error");
  kill(getppid(), SIGUSR2);

  signal(SIGUSR1, urlOpener);

  for (urlCount = 0; urlCount < *numUrls; urlCount++) {
    sigpause(SIGUSR1);
    signal(SIGUSR1, urlOpener);
  }

}

static void launcher(void)
{
  int err, pid;
  FILE *fp;
  
  /* should first check if netscape is already running */  
  pid = findproc("netscape");

  if (!pid) {
    procFlag = 0;   /* netscape not found */
    err = system("/usr/local/bin/netscape2");
    if (err == -1)
      Abandon( "system command failure in launcher");
    fprintf(stdout, "netscape finished\n");
  }
  else {
    procFlag = 1; /* netscape found */
    fprintf(stdout, "Netscape running already\n");
  }
  
}

static int findproc(char *proc)
{
  int err, pid;
  FILE *fp;
  char command[256];

  strcpy(command, "ps -e | grep ");
  strcat(command, proc);
  strcat(command, " > .result");
  
  err = system(command);
  if (err == -1)
    Abandon("launcher error");
  
  fp = fopen(".result", "r");
  if (!fp)
    Abandon("launcher: file open error");
  
  fscanf(fp, "%d", &pid);

  return pid;
}

static void urlOpener(int signo)
{
  char command[256];
  int err;

  printf(" play: recceived signal # %d\n", signo);

  strcpy(command, "netscape2 -remote ");
  strcat(command, Urls[urlCount] -> url);
  printf("command: %s\n", command);
  err = system(command);
  if (err == -1) 
    Abandon( "system command failure");
}
  

static void timer(int *numUrls)
{
  int count;
  pid_t parent;
  
  parent = getppid();

  for (count = 0; count < *numUrls; count++)
    Urls[count]->offset = Urls[count]->offset/(11025 * 2);

  for (count = 0; count < *numUrls; count++) {
    if (!count)
      sleep(Urls[count] -> offset);
    else
      sleep(Urls[count] -> offset - Urls[count-1] -> offset);
    printf("timer:slept %d secs\n",Urls[count] -> offset - Urls[count-1] -> offset); 
    kill(parent, SIGUSR1);
  }
}

static ALport openPort (char *name, header *info)
{
  ALconfig audioconfig;
  ALport  audioportOutput;
  int err;
  long rate, queuesize, buf[2];
  
  switch (info -> rate) {
  case 1 : rate = 8000;
    break;
  case 2 : rate = 11025;
    break;
  case 3 : rate = 22000;
    break;
  case 4 : rate = 44100;
  }

  blockSize = (rate * 100)/1000;
  queuesize = blockSize * 10;

  ALseterrorhandler(0);
  audioconfig = ALnewconfig();

  ALsetsampfmt(audioconfig, AL_SAMPFMT_TWOSCOMP);
  ALsetwidth(audioconfig, AL_SAMPLE_16);
  ALsetqueuesize(audioconfig, queuesize);
  ALsetchannels(audioconfig, info -> channel);
   
  audioportOutput = ALopenport(name, "w", audioconfig);
  if (audioportOutput == (ALport) 0) {
    err = oserror();
    switch (err) {
    case AL_BAD_NO_PORTS : Err_mesg("System is out of audio ports"); 
      break;
    case AL_BAD_DEVICE_ACCESS : Err_mesg("Couldn't access audio device");
      break;
    case AL_BAD_OUT_OF_MEM : Err_mesg("Out of memory");
    }
    return ((ALport) -1); 
  }
  ALfreeconfig(audioconfig);

  buf[0] = AL_OUTPUT_RATE;
  buf[1] = rate;
  
  if (ALsetparams(AL_DEFAULT_DEVICE, buf, 2) < 0) 
    if (oserror() == AL_BAD_DEVICE_ACCESS)
      Abandon("couldn't access audio hardware to set params");

  return audioportOutput;
}


static void Abandon(char *s)
{
  fprintf(stderr, "Fatal Error: %s, errno = %d\n", s, errno);
  exit(1);
}

static Err_mesg(char *s)
{
  fprintf(stderr, "error: %s\n", s);
}
